#include "linear_assignment_benchmark.h"
#include <iostream>
#include <fstream>
#include <sys/types.h>
#include <dirent.h>
#include <chrono>
#include <vector>

using namespace std;
typedef chrono::high_resolution_clock Time;
typedef chrono::milliseconds ms;
typedef chrono::duration<float> fsec;

void print_help() {
    cout << "Well-formed input file is as follows" << endl;
    cout << "random algorithm eps(auction only) assigment_dimension" << endl;
    cout << "or" << endl;
    cout << "matrix algorithm eps(auction only) assigment_dimension matrix" << endl;
}

int main(int argc, char** argv) {
    if (argc != 2) {
        cerr << "Usage ./lap [input filename]\n";
        return 0;
    }

    ifstream infile(argv[1]);

    if (!infile.is_open()) {
        cerr << "Input file cannot be found.\n";
        return 1;
    }

    auto time_state = Time::now();

    string method;
    infile >> method;
    string alg;
    string temp;
    infile >> alg;
    infile >> temp;
    int eps = 1;
    if (alg == "auction") {
        eps = stod(temp);
        infile >> temp;
    }
    int dim  = stod(temp);

    vector<vector<double> > cost_matrix(dim, vector<double>(dim));

    if (method == "random") {
        srand(1);

        for (int i = 0; i < dim; i++) {
            for (int j = 0; j < dim; j++ ) {
                cost_matrix[i][j] = rand() % 10000;
            }
        }
    } else if (method == "matrix") {
        for (int i = 0; i < dim; i++) {
            for (int j = 0; j < dim; j++ ) {
                infile >> temp;
                cost_matrix[i][j] = stod(temp);
            }
        }
    } else {
        cout << "Method: " << method << " does not exist" << endl;
        print_help();
        return 0;
    }

    if (alg == "hungarian" || alg == "auction" || alg == "jonker-volgenant") {
        RunAssignmentBench(alg, cost_matrix, eps);
    } else {
        cout << "Algorithm: " << alg << " does not exist" << endl;
        print_help();
    }

    fsec total_time = (Time::now() - time_state);
    cout << "Algorithm: " << alg;    
    if (alg == "auction") {
        cout << " with eps value of " << eps; 
    }
    cout << "\n";
    cout << "On a cost matrix of size " << cost_matrix.size() <<" x " << cost_matrix[0].size() << "\n";    
    cout << "Total time taken: " << total_time.count() << "s\n";  

    return 0;
}
